cls
pushd (Split-Path -Path $MyInvocation.MyCommand.Definition -Parent)
$ErrorActionPreference = "Stop"

#theme info
$themeName	= "TheBootstrapTheme"
$themeID  	= "89c03151-5827-481f-90a7-9372857e84a9"
$appPackageName = "thebootstraptheme.spfx.sppkg"
$appName = "{0}-client-side-solution" -f $themeName.ToLower()
$spcolorFileName = "{0}.spcolor" -f $themeName.ToLower()
$global:relativeWebUrl = ""

#Path to the theme gallery
$targetDir = "/_catalogs/theme/15"
$spcolorPath = $null
if(Test-Path (".\{0}" -f $spcolorFileName)){
	$spcolorPath = Resolve-path (".\{0}" -f $spcolorFileName)
}


function Menu(){
	try{
		checkPnPCmdlets
	
		Write-Host ("`n#--- Installing {0} Theme ---#`n" -f $themeName) 
		
		Write-Host "1 - Install/Update theme globally and apply to a site collection"
		Write-Host "2 - Install/Update theme globally with full color experience and apply to a site collection (Classic sites w/ Modern)`n"
		Write-Host "3 - Apply theme to a site collection"
		Write-Host "4 - Apply theme with full color experience to a site collection (Classic sites w/ Modern)`n"
		Write-Host "5 - Deactivate theme from a site collection"	
		Write-Host "6 - Uninstall theme globally`n"
		Write-Host "7 - Open BindTuning documentation page"
		Write-Host "0 - Exit"	

		[string] $option="0"
		$option = Read-Host "`nSelect option"
				
		switch($option){
			"1" { connectTo; installGlobally $false; Disconnect-PnPOnline; break}
			"2" { connectTo; installGlobally $true; Disconnect-PnPOnline; break}
			"3" { connectTo; applyToSite $false; Disconnect-PnPOnline; break}
			"4" { connectTo; applyToSite $true; Disconnect-PnPOnline; break}
			"5" { connectTo; deactivateFromSite; Disconnect-PnPOnline; break}
			"6" { connectTo; uninstallGlobally; Disconnect-PnPOnline; break}
			"7" { (New-Object -Com Shell.Application).Open("http://bindtuning-spfx-themes.readthedocs.io/en/latest/SETUP/Set%20Up%20The%20Environment/"); cls; Menu; break }
			"0" { exit }
			default { cls; Menu }
		}
		
	} catch {
		Write-Host $_ -ForegroundColor Red
		closeWindow
	}

	Write-Host "Process completed."
	closeWindow
}

function connectTo(){
	$url = Read-Host "`nPlease enter the SharePoint Online site collection url"
	Write-Host "- Connecting to: $url ... " -NoNewline
	#- Connecto to site
	Connect-PnPOnline -UseWebLogin -Url $url
	#- Get the relative url of the site collection 
	$global:relativeWebUrl = (Get-PnPWeb).ServerRelativeUrl
	Write-Host "Done" -BackgroundColor Green -ForegroundColor Black
}

#option 1
#option 2
function installGlobally($fullExperience){
	#- Add or Update the theme
	$sppkgPath = Resolve-path (".\{0}" -f $appPackageName)
	Write-Host "* Checking if $appName exists on AppCatalog ... "
	$appAdded = Get-PnPApp | ? { $_.Title -eq $appName }
	if ($appAdded -eq $null) {
	    Write-Host "- Adding $appName to AppCatalog ... " -NoNewline
	    Add-PnPApp -Path $sppkgPath | Out-Null
	}else{
	    Write-Host "- Updating $appName on AppCatalog ... " -NoNewline
	    Update-PnPApp -Identity $appAdded.id
	}
	Write-Host "Done" -BackgroundColor Green -ForegroundColor Black
	
	#-Publish Theme
	$appAdded = Get-PnPApp | ? { $_.Title -eq $appName }
	if ($appAdded -ne $null) {		
	    Write-host "- Publishing it for all site collections ... " -NoNewline
	    Publish-PnPApp -Identity $appAdded.id -SkipFeatureDeployment
	    Write-Host "Done (It is now available to use on any site.)" -BackgroundColor Green -ForegroundColor Black
	} else {
		Write-Host "- Unable to get info of the installed $appName. Please retry installing the theme." -ForegroundColor Red
		closeWindow
	}
	
	#- Add Custom action
	Write-host "- Setting custom action ... " -NoNewline
	Add-PnPCustomAction -ClientSideComponentId $themeID -Name "$themeName Theme" -Title "$themeName Theme" -Location ClientSideExtension.ApplicationCustomizer -Scope site
	Write-host "Done" -BackgroundColor Green -ForegroundColor Black
	
	if($fullExperience){
		installApplySPColorExperience $true
	}
}

#option 3
#option 4
function applyToSite($fullExperience){
	#- Add Custom action
	Write-Host "* Checking if $appName exists on AppCatalog ... "
	$appAdded = Get-PnPApp | ? { $_.Title -eq $appName }
	if ($appAdded) {		
	Write-host "- Setting custom action ... " -NoNewline
	Add-PnPCustomAction -ClientSideComponentId $themeID -Name "$themeName Theme" -Title "$themeName Theme" -Location ClientSideExtension.ApplicationCustomizer -Scope site
		Write-host "Done" -BackgroundColor Green -ForegroundColor Black
	}else{
	    Write-Host ("{0} not installed on the site collection" -f $appName) -ForegroundColor Red
		Write-Host ("Please use option 1 or option 2 to install it first." -f $appName)
	}
	
	if($fullExperience){
		installApplySPColorExperience $false
	}
}

#install full color experience
function installApplySPColorExperience($install){
	if($install){
		#- Upload the theme color schema to the themes library
		if($spcolorPath){
			Write-host "- Uploading the theme color file ... " -NoNewline
			try{
			Add-PnPFile -Path $spcolorPath -Folder $targetDir
				Write-host "Done" -BackgroundColor Green -ForegroundColor Black
			} catch {
				Write-Host ("Unable to install {0}" -f $spcolorFileName) -ForegroundColor Red
				Write-Host $_.Exception.Message -ForegroundColor Red
			}
		} else {
			Write-Host ("- Missing {0} on the folder" -f $spcolorFileName) -ForegroundColor Red
		}
	}
	
	#check if spcolor is installed
	$installedSPColor = Find-PnPFile -Folder ("{0}/_catalogs/theme/15" -f $global:relativeWebUrl) -Match $spcolorFileName
	if($installedSPColor){
	#- Register the theme on the Composed Looks list
	Write-host "- Registering the theme color schema ... " -NoNewline
	Add-PnPListItem -List "Composed Looks" -ContentType "Item" -Values @{"Title" = $themeName; "Name" = $themeName; "MasterPageUrl" = ("{0}/_catalogs/masterpage/seattle.master, {0}/_catalogs/masterpage/seattle.master" -f $global:relativeWebUrl); "ThemeUrl" = ("{0}/_catalogs/theme/15/{1}, {0}/_catalogs/theme/15/{1}" -f $global:relativeWebUrl, $spcolorFileName); "DisplayOrder" = "1"}
		Write-host "Done" -BackgroundColor Green -ForegroundColor Black

	#- Reset all sites in the site collection to use the uploaded theme
	write-host "- Applying the theme color schema ... " -NoNewline
	$pathToColorFile = "{0}/_catalogs/theme/15/{1}" -f $global:relativeWebUrl, $spcolorFileName
	Set-PnPTheme -ColorPaletteUrl $pathToColorFile -ResetSubwebsToInherit
		Write-host "Done" -BackgroundColor Green -ForegroundColor Black
	} else {
		Write-Host ("{0} not installed on {1}/_catalogs/theme/15" -f $spcolorFileName, $global:relativeWebUrl) -ForegroundColor Red
		Write-Host ("Skipping applying full color experience." -f $spcolorFileName, $global:relativeWebUrl) -ForegroundColor Red
	}
}

#option 5
function deactivateFromSite(){
	Write-host "- Deactivating the theme ... " -NoNewline
	$customActions = Get-PnPCustomAction -Scope site
	foreach ($customAction in $customActions){
		if($customAction.Name -eq ("{0} Theme" -f $themeName)){
			Remove-PnPCustomAction -scope site -identity $customAction.Id -Force
			break 
		}
	}
	Write-host "Done" -BackgroundColor Green -ForegroundColor Black
}

#option 6
function uninstallGlobally(){
	
	#- Deactivate on all sites
	Write-host "- Deactivating the theme ... " -NoNewline
	$customActions = Get-PnPCustomAction -Scope site
	foreach ($customAction in $customActions){
		if($customAction.Name -eq $themeName){
			Remove-PnPCustomAction -identity $customAction.Id -Force
			break
		}
	}
	Write-host "Done" -BackgroundColor Green -ForegroundColor Black
	
	#- Removing theme color shema
#	Write-Host "-  Uninstalling theme color  ... " -NoNewline
#	$relativeFilePath = "{0}{1}/{2}" -f $global:relativeWebUrl,$targetDir,$spcolorFileName
# 	Remove-PnPFile -ServerRelativeUrl $relativeFilePath -Recycle -Force
#	Write-Host "Done" -BackgroundColor Green -ForegroundColor Black
	
	#- Remove the theme
	Write-Host "* Checking if $appName exists on AppCatalog ... "
	$appAdded = Get-PnPApp | ? { $_.Title -eq $appName }
	if ($appAdded -eq $null) {
	    Write-Host "- Not able to find the $appName on AppCatalog to remove"
	}else{
	    Write-Host "- Removing $appName from AppCatalog ... " -NoNewline
	    Remove-PnPApp -Identity $appAdded.id
		Write-Host "Done" -BackgroundColor Green -ForegroundColor Black
	}
}

function closeWindow(){
	Write-Host "Press any key to close this window."
	$null = $Host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
	exit
}

function checkPnPCmdlets(){
	$showPNPMenu = $false
	
	#check powershell version
	$PSversion = $PSVersionTable.PSVersion
	if(!$PSVersion){
		cls
		Write-Host "`n* Unable to check installed PowerShell version (Requires Windows Management Framework v4.0 installed)." -ForegroundColor Red
		$showPNPMenu = $true
	} elseif($PSVersion.Major -lt 4) { 
		cls
		Write-Host ("`n* The installed PowerShell version is lower than the requirement (required: 4 or above , installed: {0})." -f $PSVersion) -ForegroundColor Red
		$showPNPMenu = $true
	}
	
	#Check PnP Cmdlets
	$PNPCmdlets = Get-Module SharePointPnPPowerShellOnline -ListAvailable | Select-Object Name,Version | Sort-Object Version -Descending
	if(!$PNPCmdlets){
		cls
		Write-Host ("`n* SharePoint Patterns and Practices (PnP) Cmdlets are not installed.") -ForegroundColor Red
		$showPNPMenu = $true
	}
	
	if($showPNPMenu){
		Write-Host ("`nPlease, check the requirements before proceding with the installation.") -ForegroundColor Red

		# Menu
		Write-Host ("`n#--- SharePoint Patterns and Practices (PnP) Cmdlets ---#`n") 

		Write-Host "1 - Open BindTuning documentation page"
		Write-Host "2 - Open PnP Cmdlets documentation page"
		Write-Host "3 - Open PnP Cmdlets download page. (Download ""SharePointPnPPowerShellOnline.msi"" and install or use Option 4)"
		Write-Host "4 - Install SharePoint PnP PowerShell *Online* Cmdlets. (need to Run this script first as Administrator)"		
		Write-Host "5 - Open Windows Management Framework v4.0 download page"
		Write-Host "0 - Exit"

		[string] $option="0"
		$option = Read-Host "`nSelect option"

		switch($option){
			"1" { (New-Object -Com Shell.Application).Open("http://bindtuning-spfx-themes.readthedocs.io/en/latest/SETUP/Installation/"); checkPnPCmdlets; break }
			"2" {  	
				  (New-Object -Com Shell.Application).Open("https://github.com/SharePoint/PnP-PowerShell#sharepointpnppowershell-commands")
				  (New-Object -Com Shell.Application).Open("https://docs.microsoft.com/en-us/powershell/sharepoint/sharepoint-pnp/sharepoint-pnp-cmdlets")
				  checkPnPCmdlets 
				  break
				}
			"3" { (New-Object -Com Shell.Application).Open("https://github.com/SharePoint/PnP-PowerShell/releases/"); checkPnPCmdlets; break }
			"4" { cls; Invoke-Expression (New-Object Net.WebClient).DownloadString('https://raw.githubusercontent.com/sharepoint/PnP-PowerShell/master/Samples/Modules.Install/Install-SharePointPnPPowerShell.ps1'); Menu; break}
			"5" { (New-Object -Com Shell.Application).Open('https://www.microsoft.com/en-us/download/details.aspx?id=40855'); checkPnPCmdlets; break }
			"0" { exit }
			default { cls; Menu }
		}
	}
}

& Menu
